/* This file is part of tagr.
   Copyright (C) 2006, 2009 Sergey Poznyakoff

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA. */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <ctype.h>
#include <errno.h>
#include <gd.h>
#include <gdfonts.h>

#include <tagr.h>

int fill_incoming_option = 1; /* Fill incoming graph */
int percent_option = 0;
int transparent_option = 1;
int zero_unknown_option = 1;

int color_background[3]    = { 245,245,245 };
int color_light[3]         = { 194,194,194 };
int color_dark[3]          = { 100,100,100 };
int color_major[3]         = { 255,0,0 };
int color_in[3]            = { 0,235,12 };
int color_out[3]           = { 0,94,255 };
int color_grid[3]          = { 0,0,0 };
int color_in_max[3]        = { 0,166,33 };
int color_out_max[3]       = { 255,0,255 };
int color_percent[3]       = { 239,159,79 };

int graph_xsize = 460;
int graph_ysize = 100;

static char *short_suffix[] = {"", "k", "M", "G", "T"};
char **number_suffix = short_suffix;
size_t number_suffix_count = sizeof (short_suffix) / sizeof (short_suffix[0]);

#define make_color_index(g, ar) \
  gdImageColorAllocate (g, (ar)[0], (ar)[1], (ar)[2])

int
draw_graph (FILE *fp,
	    queue_t *dataq, const struct avg_acc *avg, time_t now,
	    int xstep, unsigned long xmax, unsigned long ymax,
	    int growright)
{
  int x;
  int i, n;
  gdImagePtr graph, brush_out, brush_outm, brush_outp;
  int i_background, i_light, i_dark;
  int i_major, i_in, i_out, i_grid, i_inm, i_outm;
  int i_outp, i_outpg;
  double xscale, yscale;

  yscale = ((double) graph_ysize - 35) / ymax;
  xscale = ((double) graph_xsize - 100 - percent_option * 30) / xmax;
  
#define ytr(y)			    \
  (unsigned long) ((ymax - (y)) * yscale + 14)
#define xtr(x)			    \
  (unsigned long) (growright ? ((graph_xsize - (x))) : (x))

  graph = gdImageCreate (graph_xsize, graph_ysize);
  brush_out = gdImageCreate (1, 2);
  brush_outm = gdImageCreate (1, 2);
  brush_outp = gdImageCreate (1, 2);

  i_background = make_color_index (graph, color_background);
  i_light = make_color_index (graph, color_light);
  i_dark = make_color_index (graph, color_dark);

  if (transparent_option)
    gdImageColorTransparent (graph, i_background);

  gdImageInterlace (graph, 1);
  
  i_major = make_color_index (graph, color_major);
  i_in = make_color_index (graph, color_in);
  i_out = make_color_index (brush_out, color_out);
  i_grid = make_color_index (graph, color_grid);
  i_inm = make_color_index (graph, color_in_max);
  i_outm = make_color_index (brush_outm, color_out_max);
  i_outp = make_color_index (brush_outp, color_percent);
  i_outpg = make_color_index (graph, color_percent);

  /* Draw the image border */
  gdImageLine (graph, 0, 0, graph_xsize - 1, 0, i_light);
  gdImageLine (graph, 1, 1, graph_xsize - 2, 1, i_light);
  gdImageLine (graph, 0, 0, 0, graph_ysize - 1, i_light);
  gdImageLine (graph, 1, 1, 1, graph_ysize - 2, i_light);
  gdImageLine (graph, graph_xsize - 1, 0, graph_xsize - 1,
	       graph_ysize - 1, i_dark);
  gdImageLine (graph, 0, graph_ysize - 1, graph_xsize - 1,
	       graph_ysize - 1, i_dark);
  gdImageLine (graph, graph_xsize - 2, 1, graph_xsize - 2,
	       graph_ysize - 2, i_dark);
  gdImageLine (graph, 1, graph_ysize - 2, graph_xsize - 2,
	       graph_ysize - 2, i_dark);

  n = queue_count (dataq);
  for (i = n - 1, x = 0; i > 0; i--, x++)
    {
      struct traffic_history *th = queue_get_ptr (dataq, i);
      struct traffic_history *tnext = queue_get_ptr (dataq, i - 1);
      if (fill_incoming_option)
	gdImageLine (graph,
		     xtr (x), ytr (0),
		     xtr (x), ytr (tnext->inrate), i_in);
      gdImageLine (graph, xtr (x), ytr (th->inrate),
		   xtr (x+1), ytr (tnext->inrate), i_in);
#if 0
      gdImageLine (graph, x + 0.5, ytr (0),
		   x + 0.5, ytr (th->inrate), i_in);
#endif
    }

  /* FIXME: Drow outgoing traffic, grids and legends */
  gdImagePng (graph, fp);
  
  gdImageDestroy (graph);
  gdImageDestroy (brush_out);
  gdImageDestroy (brush_outm);
  gdImageDestroy (brush_outp);
  return 0;
}
